'''This tool creates service areas around points using ArcGIS Network Analyst extension of ArcGIS Desktop'''

import os
import sys

import arcpy

def main():
    '''Program entry point'''

    #Check out the Network Analyst extension license
    arcpy.CheckOutExtension("Network")

    #Get the name and version of the product used to run the script
    install_info = arcpy.GetInstallInfo()
    product_version = "{0} {1}".format(install_info["ProductName"], install_info["Version"])

    ##Setup inputs 
    #Use the Stores feature class in a file geodatabase called inputs in the same folder as the script
    cwd = os.path.dirname(os.path.abspath(__file__))
    input_gdb = os.path.join(cwd, "inputs.gdb")
    facilities = os.path.join(input_gdb, "Stores")
    #Use the Streets_ND network dataset in the Transportation feature dataset
    network_dataset = os.path.join(input_gdb, "Transportation", "Streets_ND")
    #Setup other inputs for creating service area layer
    break_values = "2 4 6"
    travel_modes = ["Driving Time", "Walking Time"]
    impedance_attribute = "TravelTime"
    search_tolerance = "20000 Meters"
    service_area_layer_name = "ServiceAreas"

    #Store outputs in a file geodatabase called outputs in the same folder as the script
    #Create the file geodatabase if it does not exist
    output_gdb_name = "outputs.gdb"
    output_gdb = os.path.join(cwd, output_gdb_name)
    if not os.path.exists(output_gdb):
        arcpy.management.CreateFileGDB(cwd, output_gdb_name)

    #Set environment settings
    arcpy.env.workspace = output_gdb
    arcpy.env.overwriteOutput = True

    #Create a new service area layer
    result = arcpy.na.MakeServiceAreaLayer(network_dataset, service_area_layer_name, impedance_attribute,
                                           "TRAVEL_FROM", break_values)
    #Get the layer object from the result object. The service area layer can now be referenced using the layer object.
    service_area_layer = result.getOutput(0)

    #Get the names of all the sublayers within the service area layer.
    sub_layer_names = arcpy.na.GetNAClassNames(service_area_layer)

    #Stores the layer names that we will use later
    facilities_layer_name = sub_layer_names["Facilities"]
    polygons_layer_name = sub_layer_names["SAPolygons"]

    #Get the layer object for the output polygons. 
    if hasattr(service_area_layer, "listLayers"):
        #When using ArcGIS Pro, get the polygons sub layer using the listLayers method on the service area
        #layer object
        polygons_layer = service_area_layer.listLayers(polygons_layer_name)[0]
    else:
        #When using ArcMap, get the polygons sub layer using the ListLayers function in the mapping module
        polygons_layer = arcpy.mapping.ListLayers(service_area_layer, polygons_layer_name)[0]

    #Load the stores as facilities. Map the StoreName field from stores as Name field using field mappings
    field_mappings = arcpy.na.NAClassFieldMappings(service_area_layer, facilities_layer_name)
    field_mappings["Name"].mappedFieldName = "StoreName"
    arcpy.na.AddLocations(service_area_layer, facilities_layer_name, facilities, field_mappings, search_tolerance)

    #Get a list of travel modes available in the network dataset
    network_dataset_travel_modes = arcpy.na.GetTravelModes(network_dataset)
    #Update the travel mode settings for each travel mode and solve the layer
    service_area_layer_properties = arcpy.na.GetSolverProperties(service_area_layer)
    for travel_mode in travel_modes:
        if not travel_mode in network_dataset_travel_modes:
            raise arcpy.ExecuteError("{0} travel mode does not exist in {1}".format(travel_mode, network_dataset))
        service_area_layer_properties.applyTravelMode(network_dataset_travel_modes[travel_mode])
        #Solve the service area layer
        arcpy.AddMessage("Generating '{0}' minutes service areas using {1} travel mode with {2}".format(break_values,
                                                                                                        travel_mode,
                                                                                                        product_version))
        #Fail if the solve returned a failure
        try:
            arcpy.na.Solve(service_area_layer)
        except arcpy.ExecuteError as ex:
            arcpy.AddError(arcpy.GetMessages(2))
            raise arcpy.ExecuteError("An error occured when running the tool")

        #Save the results to a file geodatabase feature class.
        output_service_area_name = u"Desktop_{0}".format(arcpy.ValidateTableName(travel_mode, output_gdb))
        arcpy.management.CopyFeatures(polygons_layer, output_service_area_name)

if __name__ == "__main__":
    try:
        main()
    except Exception as ex:
        print(ex.message)